@@ -40,8 +40,8 @@
  *
  * Examples for caps features would be the requirement of a specific #GstMemory
  * types or the requirement of having a specific #GstMeta on the buffer. Features
- * are given as a string of the format "memory:GstMemoryTypeName" or
- * "meta:GstMetaAPIName".
+ * are given as a string of the format `memory:GstMemoryTypeName` or
+ * `meta:GstMetaAPIName`.
  *
  * Since: 1.2
  */
@@ -171,8 +171,6 @@ gst_caps_feature_name_is_valid (const gchar * feature)
  *
  * Creates a new, empty #GstCapsFeatures.
  *
- * Free-function: gst_caps_features_free
- *
  * Returns: (transfer full): a new, empty #GstCapsFeatures
  *
  * Since: 1.2
@@ -182,7 +180,7 @@ gst_caps_features_new_empty (void)
 {
   GstCapsFeatures *features;
 
-  features = g_slice_new (GstCapsFeatures);
+  features = g_new (GstCapsFeatures, 1);
   features->type = _gst_caps_features_type;
   features->parent_refcount = NULL;
   features->array = g_array_new (FALSE, FALSE, sizeof (GQuark));
@@ -200,8 +198,6 @@ gst_caps_features_new_empty (void)
  * to any other #GstCapsFeatures but caps with these are
  * unfixed.
  *
- * Free-function: gst_caps_features_free
- *
  * Returns: (transfer full): a new, ANY #GstCapsFeatures
  *
  * Since: 1.2
@@ -217,6 +213,27 @@ gst_caps_features_new_any (void)
   return features;
 }
 
+/**
+ * gst_caps_features_new_single:
+ * @feature: The feature
+ *
+ * Creates a new #GstCapsFeatures with a single feature.
+ *
+ * Returns: (transfer full): a new #GstCapsFeatures
+ *
+ * Since: 1.20
+ */
+GstCapsFeatures *
+gst_caps_features_new_single (const gchar * feature)
+{
+  GstCapsFeatures *features;
+  g_return_val_if_fail (feature != NULL, NULL);
+
+  features = gst_caps_features_new_empty ();
+  gst_caps_features_add (features, feature);
+  return features;
+}
+
 /**
  * gst_caps_features_new:
  * @feature1: name of first feature to set
@@ -225,8 +242,6 @@ gst_caps_features_new_any (void)
  * Creates a new #GstCapsFeatures with the given features.
  * The last argument must be %NULL.
  *
- * Free-function: gst_caps_features_free
- *
  * Returns: (transfer full): a new, empty #GstCapsFeatures
  *
  * Since: 1.2
@@ -253,8 +268,6 @@ gst_caps_features_new (const gchar * feature1, ...)
  *
  * Creates a new #GstCapsFeatures with the given features.
  *
- * Free-function: gst_caps_features_free
- *
  * Returns: (transfer full): a new, empty #GstCapsFeatures
  *
  * Since: 1.2
@@ -284,8 +297,6 @@ gst_caps_features_new_valist (const gchar * feature1, va_list varargs)
  * Creates a new #GstCapsFeatures with the given features.
  * The last argument must be 0.
  *
- * Free-function: gst_caps_features_free
- *
  * Returns: (transfer full): a new, empty #GstCapsFeatures
  *
  * Since: 1.2
@@ -312,8 +323,6 @@ gst_caps_features_new_id (GQuark feature1, ...)
  *
  * Creates a new #GstCapsFeatures with the given features.
  *
- * Free-function: gst_caps_features_free
- *
  * Returns: (transfer full): a new, empty #GstCapsFeatures
  *
  * Since: 1.2
@@ -343,7 +352,7 @@ gst_caps_features_new_id_valist (GQuark feature1, va_list varargs)
  * Sets the parent_refcount field of #GstCapsFeatures. This field is used to
  * determine whether a caps features is mutable or not. This function should only be
  * called by code implementing parent objects of #GstCapsFeatures, as described in
- * the MT Refcounting section of the design documents.
+ * [the MT refcounting design document](additional/design/MT-refcounting.md).
  *
  * Returns: %TRUE if the parent refcount could be set.
  *
@@ -380,8 +389,6 @@ gst_caps_features_set_parent_refcount (GstCapsFeatures * features,
  *
  * Duplicates a #GstCapsFeatures and all its values.
  *
- * Free-function: gst_caps_features_free
- *
  * Returns: (transfer full): a new #GstCapsFeatures.
  *
  * Since: 1.2
@@ -424,7 +431,7 @@ gst_caps_features_free (GstCapsFeatures * features)
 #endif
   GST_TRACE ("free caps features %p", features);
 
-  g_slice_free (GstCapsFeatures, features);
+  g_free (features);
 }
 
 /**
@@ -434,15 +441,14 @@ gst_caps_features_free (GstCapsFeatures * features)
  * Converts @features to a human-readable string representation.
  *
  * For debugging purposes its easier to do something like this:
- * |[<!-- language="C" -->
+ *
+ * ``` C
  * GST_LOG ("features is %" GST_PTR_FORMAT, features);
- * ]|
+ * ```
+ *
  * This prints the features in human readable form.
  *
- * Free-function: g_free
- *
  * Returns: (transfer full): a pointer to string allocated by g_malloc().
- *     g_free() after usage.
  *
  * Since: 1.2
  */
@@ -489,11 +495,8 @@ priv_gst_caps_features_append_to_gstring (const GstCapsFeatures * features,
  *
  * Creates a #GstCapsFeatures from a string representation.
  *
- * Free-function: gst_caps_features_free
- *
  * Returns: (transfer full) (nullable): a new #GstCapsFeatures or
- *     %NULL when the string could not be parsed. Free with
- *     gst_caps_features_free() after use.
+ *     %NULL when the string could not be parsed.
  *
  * Since: 1.2
  */
@@ -649,7 +652,7 @@ gst_caps_features_get_nth_id (const GstCapsFeatures * features, guint i)
  * @features: a #GstCapsFeatures.
  * @feature: a feature
  *
- * Check if @features contains @feature.
+ * Checks if @features contains @feature.
  *
  * Returns: %TRUE if @features contains @feature.
  *
@@ -671,7 +674,7 @@ gst_caps_features_contains (const GstCapsFeatures * features,
  * @features: a #GstCapsFeatures.
  * @feature: a feature
  *
- * Check if @features contains @feature.
+ * Checks if @features contains @feature.
  *
  * Returns: %TRUE if @features contains @feature.
  *
@@ -705,7 +708,7 @@ gst_caps_features_contains_id (const GstCapsFeatures * features, GQuark feature)
  * @features1: a #GstCapsFeatures.
  * @features2: a #GstCapsFeatures.
  *
- * Check if @features1 and @features2 are equal.
+ * Checks if @features1 and @features2 are equal.
  *
  * Returns: %TRUE if @features1 and @features2 are equal.
  *
@@ -751,7 +754,7 @@ gst_caps_features_is_equal (const GstCapsFeatures * features1,
  * gst_caps_features_is_any:
  * @features: a #GstCapsFeatures.
  *
- * Check if @features is %GST_CAPS_FEATURES_ANY.
+ * Checks if @features is %GST_CAPS_FEATURES_ANY.
  *
  * Returns: %TRUE if @features is %GST_CAPS_FEATURES_ANY.
  *
