@@ -26,12 +26,55 @@
 #define GST_BYTE_READER_DISABLE_INLINES
 #include "gstbytereader.h"
 
+#include "gst/glib-compat-private.h"
 #include <string.h>
 
 /**
  * SECTION:gstbytereader
+ * @title: GstByteReader
  * @short_description: Reads different integer, string and floating point
  *     types from a memory buffer
+ * @symbols:
+ * - gst_byte_reader_skip_unchecked
+ * - gst_byte_reader_get_uint8_unchecked
+ * - gst_byte_reader_peek_uint8_unchecked
+ * - gst_byte_reader_get_int8_unchecked
+ * - gst_byte_reader_peek_int8_unchecked
+ * - gst_byte_reader_get_uint16_le_unchecked
+ * - gst_byte_reader_get_uint16_be_unchecked
+ * - gst_byte_reader_peek_uint16_le_unchecked
+ * - gst_byte_reader_peek_uint16_be_unchecked
+ * - gst_byte_reader_get_int16_le_unchecked
+ * - gst_byte_reader_get_int16_be_unchecked
+ * - gst_byte_reader_peek_int16_le_unchecked
+ * - gst_byte_reader_peek_int16_be_unchecked
+ * - gst_byte_reader_get_uint24_le_unchecked
+ * - gst_byte_reader_get_uint24_be_unchecked
+ * - gst_byte_reader_peek_uint24_le_unchecked
+ * - gst_byte_reader_peek_uint24_be_unchecked
+ * - gst_byte_reader_get_int24_le_unchecked
+ * - gst_byte_reader_get_int24_be_unchecked
+ * - gst_byte_reader_peek_int24_le_unchecked
+ * - gst_byte_reader_peek_int24_be_unchecked
+ * - gst_byte_reader_get_uint32_le_unchecked
+ * - gst_byte_reader_get_uint32_be_unchecked
+ * - gst_byte_reader_peek_uint32_le_unchecked
+ * - gst_byte_reader_peek_uint32_be_unchecked
+ * - gst_byte_reader_get_int32_le_unchecked
+ * - gst_byte_reader_get_int32_be_unchecked
+ * - gst_byte_reader_peek_int32_le_unchecked
+ * - gst_byte_reader_peek_int32_be_unchecked
+ * - gst_byte_reader_get_float32_le_unchecked
+ * - gst_byte_reader_get_float32_be_unchecked
+ * - gst_byte_reader_get_float64_le_unchecked
+ * - gst_byte_reader_get_float64_be_unchecked
+ * - gst_byte_reader_peek_float32_le_unchecked
+ * - gst_byte_reader_peek_float32_be_unchecked
+ * - gst_byte_reader_peek_float64_le_unchecked
+ * - gst_byte_reader_peek_float64_be_unchecked
+ * - gst_byte_reader_peek_data_unchecked
+ * - gst_byte_reader_get_data_unchecked
+ * - gst_byte_reader_dup_data_unchecked
  *
  * #GstByteReader provides a byte reader that can read different integer and
  * floating point types from a memory buffer. It provides functions for reading
@@ -56,7 +99,7 @@
 GstByteReader *
 gst_byte_reader_new (const guint8 * data, guint size)
 {
-  GstByteReader *ret = g_slice_new0 (GstByteReader);
+  GstByteReader *ret = g_new0 (GstByteReader, 1);
 
   ret->data = data;
   ret->size = size;
@@ -76,7 +119,7 @@ gst_byte_reader_free (GstByteReader * reader)
 {
   g_return_if_fail (reader != NULL);
 
-  g_slice_free (GstByteReader, reader);
+  g_free (reader);
 }
 
 /**
@@ -769,7 +812,6 @@ GST_BYTE_READER_PEEK_GET(64,gdouble,float64_be)
  * position if at least @size bytes are left and
  * updates the current position.
  *
- *
  * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
@@ -790,7 +832,6 @@ gst_byte_reader_get_data (GstByteReader * reader, guint size,
  * position if at least @size bytes are left and
  * keeps the current position.
  *
- *
  * Returns: %TRUE if successful, %FALSE otherwise.
  */
 gboolean
@@ -921,7 +962,7 @@ _masked_scan_uint32_peek (const GstByteReader * reader,
  * Returns: offset of the first match, or -1 if no match was found.
  *
  * Example:
- * <programlisting>
+ * |[
  * // Assume the reader contains 0x00 0x01 0x02 ... 0xfe 0xff
  *
  * gst_byte_reader_masked_scan_uint32 (reader, 0xffffffff, 0x00010203, 0, 256);
@@ -938,7 +979,7 @@ _masked_scan_uint32_peek (const GstByteReader * reader,
  * // -> returns 2
  * gst_byte_reader_masked_scan_uint32 (reader, 0xffff0000, 0x02030000, 0, 4);
  * // -> returns -1
- * </programlisting>
+ * ]|
  */
 guint
 gst_byte_reader_masked_scan_uint32 (const GstByteReader * reader, guint32 mask,
@@ -955,7 +996,7 @@ gst_byte_reader_masked_scan_uint32 (const GstByteReader * reader, guint32 mask,
  * @offset: offset from which to start scanning, relative to the current
  *     position
  * @size: number of bytes to scan from offset
- * @value: pointer to uint32 to return matching data
+ * @value: (out): pointer to uint32 to return matching data
  *
  * Scan for pattern @pattern with applied mask @mask in the byte reader data,
  * starting from offset @offset relative to the current position.
@@ -1182,7 +1223,7 @@ gst_byte_reader_dup_string_utf##bits (GstByteReader * reader, type ** str) \
     *str = NULL; \
     return FALSE; \
   } \
-  *str = g_memdup (reader->data + reader->byte, size); \
+  *str = g_memdup2 (reader->data + reader->byte, size); \
   reader->byte += size; \
   return TRUE; \
 }
