@@ -28,24 +28,17 @@
 
 #ifdef G_OS_WIN32
 #include <io.h>                 /* lseek, open, close, read */
-#undef lseek
-#define lseek _lseeki64
+#undef fseek
+#define fseek _fseeki64
 #undef off_t
 #define off_t guint64
 #else
 #include <unistd.h>
 #endif
 
-#ifdef __BIONIC__               /* Android */
-#undef lseek
-#define lseek lseek64
-#undef off_t
-#define off_t guint64
-#endif
-
 #ifdef HAVE_FSEEKO
 #define FSEEK_FILE(file,offset)  (fseeko (file, (off_t) offset, SEEK_SET) != 0)
-#elif defined (G_OS_UNIX) || defined (G_OS_WIN32)
+#elif defined (G_OS_UNIX)
 #define FSEEK_FILE(file,offset)  (lseek (fileno (file), (off_t) offset, SEEK_SET) == (off_t) -1)
 #else
 #define FSEEK_FILE(file,offset)  (fseek (file, offset, SEEK_SET) != 0)
@@ -74,6 +67,7 @@ struct _GstSparseFile
   gint fd;
   FILE *file;
   gsize current_pos;
+  gboolean was_writing;
 
   GstSparseRange *ranges;
   guint n_ranges;
@@ -104,7 +98,7 @@ get_write_range (GstSparseFile * file, gsize offset)
     next = next->next;
   }
   if (result == NULL) {
-    result = g_slice_new0 (GstSparseRange);
+    result = g_new0 (GstSparseRange, 1);
     result->start = offset;
     result->stop = offset;
 
@@ -145,7 +139,7 @@ get_read_range (GstSparseFile * file, gsize offset, gsize count)
 /**
  * gst_sparse_file_new:
  *
- * Make a new #GstSparseFile backed by the file represented with @fd.
+ * Make a new #GstSparseFile
  *
  * Returns: a new #GstSparseFile, gst_sparse_file_free() after usage.
  *
@@ -156,7 +150,7 @@ gst_sparse_file_new (void)
 {
   GstSparseFile *result;
 
-  result = g_slice_new0 (GstSparseFile);
+  result = g_new0 (GstSparseFile, 1);
   result->current_pos = 0;
   result->ranges = NULL;
   result->n_ranges = 0;
@@ -187,6 +181,17 @@ gst_sparse_file_set_fd (GstSparseFile * file, gint fd)
   return file->file != NULL;
 }
 
+static void
+gst_sparse_range_free_chain (GstSparseRange * r)
+{
+  while (r != NULL) {
+    GstSparseRange *next = r->next;
+
+    g_free (r);
+    r = next;
+  }
+}
+
 /**
  * gst_sparse_file_clear:
  * @file: a #GstSparseFile
@@ -198,14 +203,11 @@ gst_sparse_file_clear (GstSparseFile * file)
 {
   g_return_if_fail (file != NULL);
 
-  if (file->file) {
-    fclose (file->file);
-    file->file = fdopen (file->fd, "wb+");
-  }
-  g_slice_free_chain (GstSparseRange, file->ranges, next);
+  gst_sparse_range_free_chain (file->ranges);
   file->current_pos = 0;
   file->ranges = NULL;
   file->n_ranges = 0;
+  file->was_writing = FALSE;
 }
 
 /**
@@ -225,8 +227,8 @@ gst_sparse_file_free (GstSparseFile * file)
     fflush (file->file);
     fclose (file->file);
   }
-  g_slice_free_chain (GstSparseRange, file->ranges, next);
-  g_slice_free (GstSparseFile, file);
+  gst_sparse_range_free_chain (file->ranges);
+  g_free (file);
 }
 
 /**
@@ -243,7 +245,7 @@ gst_sparse_file_free (GstSparseFile * file)
  * If @available is not %NULL, it will be updated with the amount of
  * data already available after the last written byte.
  *
- * Returns: The number of bytes written of 0 on error.
+ * Returns: The number of bytes written or 0 on error.
  *
  * Since: 1.4
  */
@@ -262,7 +264,10 @@ gst_sparse_file_write (GstSparseFile * file, gsize offset, gconstpointer data,
       GST_DEBUG ("seeking to %" G_GSIZE_FORMAT, offset);
       if (FSEEK_FILE (file->file, offset))
         goto error;
+    } else if (!file->was_writing) {
+      fflush (file->file);
     }
+    file->was_writing = TRUE;
     if (fwrite (data, count, 1, file->file) != 1)
       goto error;
   }
@@ -290,7 +295,7 @@ gst_sparse_file_write (GstSparseFile * file, gsize offset, gconstpointer data,
       file->write_range = NULL;
     if (file->read_range == next)
       file->read_range = NULL;
-    g_slice_free (GstSparseRange, next);
+    g_free (next);
     file->n_ranges--;
   }
   if (available)
@@ -348,7 +353,10 @@ gst_sparse_file_read (GstSparseFile * file, gsize offset, gpointer data,
           file->current_pos, offset);
       if (FSEEK_FILE (file->file, offset))
         goto error;
+    } else if (file->was_writing) {
+      fflush (file->file);
     }
+    file->was_writing = FALSE;
     res = fread (data, 1, count, file->file);
     if (G_UNLIKELY (res < count))
       goto error;
